from __future__ import annotations

import enum
import random
import time
from typing import Any, List

import requests
import streamlit as st


def search_wikipedia_ids(searchterm: str) -> List[tuple[str, Any]]:
    """
    function with list of tuples (label:str, value:any)
    """
    # you can use a nice default here
    if not searchterm:
        return []

    # search that returns a list of wiki articles in dict form
    # with information on title, id, etc
    response = requests.get(
        "http://en.wikipedia.org/w/api.php",
        params={
            "list": "search",
            "format": "json",
            "action": "query",
            "srlimit": 10,
            "limit": 10,
            "srsearch": searchterm,
        },
        timeout=5,
    ).json()["query"]["search"]

    # first element will be shown in search, second is returned from component
    return [
        (
            str(article["title"]),
            article["pageid"],
        )
        for article in response
    ]


def search(searchterm: str) -> List[str]:
    return [f"{searchterm}_{i}" for i in range(10)]


def search_rnd_delay(searchterm: str) -> List[str]:
    time.sleep(random.randint(1, 5))
    return [f"{searchterm}_{i}" for i in range(10)]


def search_enum_return(_: str):
    e = enum.Enum("FancyEnum", {"a": 1, "b": 2, "c": 3})
    return [e.a, e.b, e.c]


def search_empty_list(_: str):
    if not st.session_state.get("search_empty_list_n", None):
        st.session_state["search_empty_list_n"] = 1
        return ["a", "b", "c"]

    return []


def selection_to_text(result):
    return f"result={result} type={type(result).__name__}"


#################################
#### application starts here ####
#################################


# searchbox configurations, see __init__.py for details
# will pass all kwargs to the searchbox component
boxes = [
    dict(
        search_function=search_wikipedia_ids,
        placeholder="Search Wikipedia",
        label=search_wikipedia_ids.__name__,
        default="SOME DEFAULT",
        clear_on_submit=False,
        clearable=True,
        key=search_wikipedia_ids.__name__,
    ),
    dict(
        search_function=search,
        default=None,
        label=search.__name__,
        clear_on_submit=True,
        key=search.__name__,
    ),
    dict(
        search_function=search_rnd_delay,
        default=None,
        clear_on_submit=False,
        clearable=True,
        label=search_rnd_delay.__name__,
        key=search_rnd_delay.__name__,
    ),
    dict(
        search_function=search_enum_return,
        clear_on_submit=True,
        key=search_enum_return.__name__,
        label=search_enum_return.__name__,
    ),
    dict(
        search_function=search_empty_list,
        clear_on_submit=True,
        key=search_empty_list.__name__,
        label=search_empty_list.__name__,
    ),
    dict(
        search_function=search,
        default_options=["inital", "list", "of", "options"],
        key=f"{search.__name__}_default_options",
        label=f"{search.__name__}_default_options",
    ),
    dict(
        search_function=search,
        default_options=[("inital", "i"), ("list", "l")],
        key=f"{search.__name__}_default_options_tuple",
        label=f"{search.__name__}_default_options_tuple",
    ),
    dict(
        search_function=search,
        key=f"{search.__name__}_rerun_disabled",
        rerun_on_update=False,
        label=f"{search.__name__}_rerun_disabled",
    ),
]



from streamlit_searchbox import _list_to_options_js, _list_to_options_py


def test_list_to_options_py_single_values():
    options = [1, 2, "three", 4.0]
    assert _list_to_options_py(options) == options


def test_list_to_options_py_tuples():
    options = [(1, "one"), (2, "two"), ("3", "three")]
    assert _list_to_options_py(options) == ["one", "two", "three"]


def test_list_to_options_py_mixed():
    options = [1, (2, "two"), 3, ("4", "four")]
    assert _list_to_options_py(options) == [1, "two", 3, "four"]


def test_list_to_options_py_empty():
    options = []
    assert _list_to_options_py(options) == []


def test_list_to_options_js_single_values():
    options = [1, 2, "three", 4.0]
    expected = [
        {"label": "1", "value": 0},
        {"label": "2", "value": 1},
        {"label": "three", "value": 2},
        {"label": "4.0", "value": 3},
    ]
    assert _list_to_options_js(options) == expected


def test_list_to_options_js_tuples():
    options = [(1, "one"), (2, "two"), ("3", "three")]
    expected = [
        {"label": "1", "value": 0},
        {"label": "2", "value": 1},
        {"label": "3", "value": 2},
    ]
    assert _list_to_options_js(options) == expected


def test_list_to_options_js_mixed():
    options = [1, (2, "two"), 3, ("4", "four")]
    expected = [
        {"label": "1", "value": 0},
        {"label": "2", "value": 1},
        {"label": "3", "value": 2},
        {"label": "4", "value": 3},
    ]
    assert _list_to_options_js(options) == expected


def test_list_to_options_js_empty():
    options = []
    assert _list_to_options_js(options) == []



from __future__ import annotations

import inspect
import re
import subprocess
import time

import pytest
import requests
from playwright.sync_api import Page, expect

from tests.utils import boxes, selection_to_text


@pytest.fixture
def screenshot():
    """
    screenshot fixture that keepts track of the number of screenshots taken
    """

    i = 0

    # TODO: tests only pass when screenshots are enabled, why?
    def _screenshot(page: Page, path: str | None = None):
        nonlocal i

        caller_name = inspect.stack()[1][3]
        page.screenshot(
            path=f"tests/screenshots/{caller_name}_{path}_{i}.png",
            full_page=True,
        )
        i += 1

    yield _screenshot


def wait_for_reload(page: Page) -> None:
    """
    wait until loading spinner is gone
    """
    page.wait_for_selector(
        "[data-testid='stStatusWidget']",
        timeout=5000,
        state="detached",
    )


@pytest.fixture
def streamlit_app():
    def is_server_running():
        try:
            return (
                requests.get("http://localhost:8501/_stcore/health", timeout=1).text
                == "ok"
            )
        except Exception as _:
            return False

    assert not is_server_running(), "Streamlit app is already running"

    # Start the Streamlit app in the background
    process = subprocess.Popen(
        [
            "streamlit",
            "run",
            "example_ci.py",
            "--server.fileWatcherType=none",
            "--logger.level=error",
        ]
    )

    start_time = time.time()

    while not is_server_running():
        time.sleep(1)

        if time.time() - start_time > 30:
            assert False, "Streamlit app failed to start within 60 seconds"

    yield process

    process.terminate()

    # Check the return code; if it's 0, the app loaded without errors
    assert process.returncode in [
        0,
        None,
    ], f"Streamlit app exited with code {process.returncode}"


def test_streamlit_app_loads(streamlit_app):
    # test fixture loads the app in the background
    pass


# iterate over all searchboxes in separate streamlit app
@pytest.mark.parametrize(
    "search_function,label",
    [
        (b["search_function"], b["label"])
        for b in boxes
        if b["label"]
        not in [
            # ignore since it relies on external api calls
            "search_wikipedia_ids",
            # ignore since it's has a different behavior
            "search_rerun_disabled",
            # TODO: make custom tests for these, initial result list makes it harder
            "search_default_options",
            "search_default_options_tuple",
        ]
    ],
)
def test_e2e(streamlit_app, search_function, label, screenshot, page: Page):
    page.goto("localhost:8501")

    expect(page).to_have_title(re.compile("Searchbox Demo"))

    wait_for_reload(page)

    screenshot(page, label)

    print(f"search_function={search_function.__name__}, search_label={label}")

    # find frame with searchbox, otherwise content isn't available
    searchbox_frames = [
        frame for frame in page.frames if "streamlit_searchbox" in frame.url
    ]

    for frame in searchbox_frames:
        l_searchbox = frame.locator(f"#{label}")

        # skip if in the wrong iframe
        if l_searchbox.count() == 0:
            continue

        search_term = "x"

        search_result = search_function(search_term)[0]

        l_searchbox.fill(search_term)
        l_searchbox.press("Enter")

        wait_for_reload(page)
        screenshot(page, label)

        l_option = frame.get_by_text(
            str(
                search_result
                if not isinstance(search_result, tuple)
                else search_result[0]
            )
        ).first
        l_option.focus()
        l_option.press("Enter")

        screenshot(page, label)

        wait_for_reload(page)

        screenshot(page, label)

        # loads result as streamlit fragment, so get from page instead of iframe
        assert (
            page.get_by_text(
                selection_to_text(
                    search_result
                    if not isinstance(search_result, tuple)
                    else search_result[1]
                )
            ).count()
            == 1
        )

        screenshot(page, label)

        break
